/*
 * IcmpJniBridge.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.jni;

import com.veraxsystems.icmp.jni.exception.IcmpFatalErrorException;
import com.veraxsystems.icmp.jni.exception.IcmpSystemException;
import com.veraxsystems.icmp.jni.exception.IcmpTimeoutException;

import java.net.InetAddress;

/**
 * The Java/DLL interface class. It offers set of Java methods that may execute set of
 * function implemented in IcmpJniBridge.DLL module.
 * Class covers ICMP socket handling in the way that each instance of IcmpJniBridge object
 * has its own ICMP socket handle. This way it is possible to create and handle many ICMP sockets
 * in application.
 * <p>
 * Underneath, the ICMP socket creates RAW socket. In Windows such operation requires
 * program being run on <i>Administrator</i> privileges.
 * <p>
 * Including this class in application causes loading of <i>IcmpJniBridge.DLL</i> file
 * that must be available in any directory specified in java <i>java.library.path</i> property.
 * <p>
 * The IcmpJniBridge object is allocating system resources when successful call to {@link #openSocket}
 * is performed. In order to release those resources the {@link #closeSocket} method must be executed.
 * 
 */
public class IcmpJniBridge 
{
   private static final long SOCKET_ERROR = -1; 
   private long handle = SOCKET_ERROR;
   
   /**
    * The maximum size of ICMP data that may be sent via IP packet
    */
   public static final int MAX_DATA_LENGTH = 65535 - 28;

   
   /**
    * Load DLL module.
    */
   public static void loadNativeLibrary()
   {
      System.loadLibrary("IcmpJniBridge");
   }

   
   /**
    * In case closesocket was not explicitly executed, do it when object is destroyed.
    */
   @Override
   protected void finalize() throws Throwable 
   {
      if (handle != SOCKET_ERROR) closeSocket();
      super.finalize();
   }
   
   
   /**
    * Get identifier unique for computer starting from 0 up to 2147483647. 
    * The identifier value is increased on every call. When maximum value is reached
    * counting is restarted from 0. 
    * <p>
    * The counter value is common for all applications using IcmpJniBridge.DLL module
    * so they never get the same value at the same time.
    * @return unique integer value
    */
   public int getId() 
   {
      return nativeGetNextMessageID();
   }
   
   
   /**
    * Open ICMP socket. If socket is already opened operation is ignored.
    * @throws IcmpSystemException exception is thrown when any system error occurs
    */
   public synchronized void openSocket() throws IcmpSystemException 
   {
      if (handle == SOCKET_ERROR) handle = nativeCreateSocket();
      return;
   } /* createSocket */
   
   
   /**
    * Close ICMP socket. If socket was not opened operation is ignored.
    * Due to release resources this method must be called after successful socket 
    * creation via {@link #openSocket}
    * @throws IcmpSystemException exception is thrown when any system error occurs
    */
   public synchronized void closeSocket() throws IcmpSystemException 
   {
      if (handle != SOCKET_ERROR) nativeCloseSocket(handle);
      handle = SOCKET_ERROR;
      return;
   } /* closeSocket */
   
   
   /**
    * Wait for ICMP socket data. 
    * Method waits until data is available for reading or given timeout is reached.
    * <p>
    * Closing socket ({@link #closeSocket}) terminates operation and causes method to throw exception.
    * @param timeout expressed in milliseconds time to wait for data
    * @throws IcmpSystemException exception is thrown when any system error occurs
    * @throws IcmpTimeoutException
    */
   public void selectSocket(int timeout) throws IcmpSystemException, IcmpTimeoutException 
   {
      nativeSelectSocket(handle, timeout);
      return;
   } /* closeSocket */
   
   
   /**
    * Send ICMP EchoRequest. 
    * Before this method is used the ICMP socket must be created using {@link #openSocket}.
    * ICMP message is sent to host specified by <code>address</code> parameter. 
    * <p>
    * The ISMP message is built using provided parameters. The <code>messageId</code> is split into ICMP message identifier and 
    * sequenceNumber (see RFC 792). The <code>data</code> is sent as ICMP optional data field.
    * <p>
    * The <code>messageId</code> should be unique value because it is used to recognize
    * response message. 
    * <p>
    * In case illegal parameter is provided or socket error occurs the <code>IcmpSystemException</code> is thrown.
    * @param address host address the request is sent to
    * @param messageId identifier of request message
    * @param data data to be sent
    * @param dataLength length of data to be send
    * @return number of bytes sent
    * @throws IcmpSystemException exception is thrown when any system error occurs
    */
   public int send(InetAddress address, int messageId, byte[] data, int dataLength) throws IcmpSystemException
   {
      int result;

      result = nativeSendEcho(handle, address.getAddress(), messageId, data, dataLength);
      return result;
   } /* closeSocket */
   
   
   /**
    * Receive ICMP response.
    * Before this method is used the ICMP socket must be created using {@link #openSocket}.
    * The received response is mapped to {@link IcmpEchoResponse} object. The response object
    * content depends an type of received ICMP response, thus, before accessing <code>IcmpEchoResponse</code> 
    * values the status of response should be verified using {@link IcmpEchoResponse#getStatus} method.
    * <p>
    * Method is a blocking operation. To introduce timeout functionality into receiving data, the {@link #selectSocket} must 
    * be called before <code>receive</code>.
    * <p>
    * Method either returns object or exception - never <code>null</code>.
    * @return received ICMP response message
    * @throws IcmpSystemException exception is thrown when any system error occurs
    * @throws IcmpFatalErrorException exception is thrown when DLL module is not able to create IcmpEchoResponse object.
    */
   public IcmpEchoResponse receive() throws IcmpSystemException, IcmpFatalErrorException
   {
      IcmpEchoResponse result;

      result = nativeReceiveMessage(handle);
      return result;
   } /* closeSocket */
   

   /*
    * The interface methods to underlying native DLL module. 
    * In order to use them the <i>IcmpJniBridge.DLL</i> must be loaded using {@link System#loadLibrary}.
    */
   private native int nativeSendEcho(long handle, byte[] destAddress, int messageId, byte[] dataBuffer, int dataLength);
   
   private native IcmpEchoResponse nativeReceiveMessage(long handle) throws IcmpSystemException;
   
   private native long nativeCreateSocket() throws IcmpSystemException;
   
   private native void nativeCloseSocket(long handle) throws IcmpSystemException;
    
   private native void nativeSelectSocket(long handle, int timeout) throws IcmpSystemException, IcmpTimeoutException;
    
   private native int nativeGetNextMessageID();
}
