/*
 * PingResult.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping;

import com.veraxsystems.icmp.IcmpStatus;
import com.veraxsystems.icmp.util.TimeUtil;

import java.net.InetAddress;

/**
 * The class is a definition of results returned by synchronous version of {@link Ping#echo Ping.echo} method.
 */
public class PingResult 
{

   private final InetAddress address;
   private final IcmpStatus status;
   private final ResultType result;
   private final long sendTime;
   private final long responseTime;

   
   /**
    * Enum defines type of result
    */
   public enum ResultType 
   {
      /**
       * Result contains ICMP response
       */
      Response(0),
      /**
       * Timeout occurred when receiving ICMP request response
       */
      Timeout(1),
      /**
       * Error in receiving data
       */
      Error(2);
      
      private int value;
      
      private ResultType(int code) 
      {
         value = code;
      }
      
      /**
       * Get integer code of enum
       * @return integer code of enum
       */
      public int getValue() 
      {
         return value;
      }
   };
   
   /**
    * Class constructor.
    * @param addressVal the address of ICMP EchoRequest destination host
    * @param statusVal status of ICMP response
    * @param resultVal type of result
    * @param sendTimeVal time when request echo was sent
    */
   public PingResult(InetAddress addressVal, IcmpStatus statusVal, ResultType resultVal, long sendTimeVal, long responseTimeVal) 
   {
      address = addressVal;
      status = statusVal;
      result = resultVal;
      sendTime = sendTimeVal;
      responseTime = responseTimeVal; //TimeUtil.currentTimeMillis();
      return;
   }

   /**
    * The {@link PingResult} is response to ICMP EchoRequest. 
    * This method get address of host the ICMP EchoRequest was sent to.
    * @return address of host request was sent to
    */
   public InetAddress getAddress() 
   {
      return address;
   }

   /**
    * Get time of request processing. It is difference between response receive time and
    * request send time.
    * @return echo processing time
    */
   public long getResponseTime() 
   {
      return responseTime - sendTime;
   }

   /**
    * Get type of result content. 
    * @return result type
    */
   public ResultType getResult() 
   {
      return result;
   }

   /**
    * Get ICMP status. The received response may be either successful EchoResponse or other 
    * kind of message. The status describes its type.
    * @return status of ICMP response
    */
   public IcmpStatus getStatus() 
   {
      return status;
   }
   
   
}
