/*
 * PingLoopTest.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.jni.handlers;

import com.veraxsystems.icmp.IcmpStatus;
import com.veraxsystems.icmp.jni.IcmpEchoResponse;
import com.veraxsystems.icmp.jni.IcmpJniBridge;
import com.veraxsystems.icmp.jni.exception.IcmpFatalErrorException;
import com.veraxsystems.icmp.jni.exception.IcmpSystemException;
import com.veraxsystems.icmp.jni.exception.IcmpTimeoutException;
import com.veraxsystems.icmp.util.TimeUtil;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

/**
 * Internally used class for running simple ICMP Echo test in many threads.
 * The send/receive operation is executed in polling mode. After send the attempt
 * to receive response is made. To avoid tight loop the receiving wait operation
 * is added (socket select)
 */
public class PingLoopTest implements Runnable 
{
   private int requestsCount;
   Counter successCounter;
   Counter failureCounter;

   
   /**
    * Initialize ICMP socket send/receive loop.
    * @param requestsCountVal number of requests to send
    * @param successCounterVal counter increased on successful execution
    * @param failureCounterVal counter increased on failure
    */
   public PingLoopTest(int requestsCountVal, Counter successCounterVal, Counter failureCounterVal) 
   {
      requestsCount = requestsCountVal;
      successCounter = successCounterVal;
      failureCounter = failureCounterVal;
   }

   public void run() 
   {
      Set<Integer> messageIdSet = new TreeSet<Integer>();
      IcmpJniBridge icmp = null;
      byte[] dataToSend = new byte[500];
      int pendingResponsesNumber = 0;
      int i = 0;
      int sent = 0;
      int received = 0;
      long beginTime;
      
      System.out.println("PingHandler.run IN " + Thread.currentThread().getId());
      try 
      {
         IcmpEchoResponse responseMsg;
         
         // The pool of addresses used in test
         InetAddress[] addreses = new InetAddress[100];
         addreses[0] = InetAddress.getByName("192.168.9.1");
         addreses[1] = InetAddress.getByName("news.yahoo.com");
         addreses[2] = InetAddress.getByName("www.google.com");
         addreses[3] = InetAddress.getByName("8.8.8.8");
         addreses[4] = InetAddress.getByName("www.vertel.com");
         addreses[5] = InetAddress.getByName("www.cnn.com");
         addreses[6] = InetAddress.getByName("www.foxnews.com");
         addreses[7] = InetAddress.getByName("www.accuweather.com");
         addreses[8] = InetAddress.getByName("www.msnbc.com");
         addreses[9] = InetAddress.getByName("en.wikipedia.org");
         addreses[10] = InetAddress.getByName("www.hostitwise.com");
         addreses[11] = InetAddress.getByName("www.advancedgaragedoorinc.com");
         addreses[12] = InetAddress.getByName("www.bidobee.com");
         addreses[13] = InetAddress.getByName("www.imedicileather.com");
         addreses[14] = InetAddress.getByName("www.rushrash.com");
         addreses[15] = InetAddress.getByName("www.usatoday.com");
         addreses[16] = InetAddress.getByName("news.bbc.co.uk");
         addreses[17] = InetAddress.getByName("www.huffingtonpost.com");
         addreses[18] = InetAddress.getByName("www.ikea.com");
         addreses[19] = InetAddress.getByName("www.veraxsystems.com");
         
         // Some data to send
         Arrays.fill(dataToSend, (byte)123); 
         
         // Use ICMP object
         icmp = new IcmpJniBridge();
         
         // Open socket
         icmp.openSocket();
         
         // Calculate hom many responses are missing
         pendingResponsesNumber = 0;
         beginTime = TimeUtil.currentTimeMillis();
         while (i < requestsCount) 
         {
            int messageId = icmp.getId();
            icmp.send((Inet4Address)addreses[i % 20], messageId, dataToSend, dataToSend.length);
            messageIdSet.add(new Integer(messageId));
            pendingResponsesNumber += 1;
            i += 1;
            sent += 1;
            try
            {
               // The first attempt to read response with timeout. If response is received the attempt
               // to read more responses in tight loop is performed
               icmp.selectSocket(5);
               responseMsg = icmp.receive();
               if (messageIdSet.remove(new Integer(responseMsg.getMessageId())))
               {
                  pendingResponsesNumber -= 1;
               }
               if (responseMsg.getStatus() != IcmpStatus.EchoReply) System.out.println("Type: " + responseMsg.getStatus());
               received += 1;
               // Now read messages as long as they are in a buffer. The timeout occurs
               // when there are no more messages
               try 
               {
                  while (true) 
                  {
                     icmp.selectSocket(1);
                     responseMsg = icmp.receive();
                     if (messageIdSet.remove(new Integer(responseMsg.getMessageId()))) 
                     {
                        pendingResponsesNumber -= 1;
                     };
                     if (responseMsg.getStatus() != IcmpStatus.EchoReply) System.out.println("Type: " + responseMsg.getStatus());
                     received += 1;
                  }
               } 
               catch (IcmpTimeoutException ex) 
               {
                  // Ignore this timeout - it is generated when input buffer is empty
                  // and we proceed with sending
               }
            }
            catch(IcmpTimeoutException ex) 
            {
               // Ignore this timeout - it is generated when input buffer is empty
               // and we proceed with sending
            }
         };
         System.out.println("Echo sent " + sent + " pending " + pendingResponsesNumber);
         System.out.println("time: " + (TimeUtil.currentTimeMillis() - beginTime));
         System.out.println("throghput: " + ((sent * dataToSend.length) / (TimeUtil.currentTimeMillis() - beginTime) * 1000) + " Bytes/sec");
         System.out.println("throghput: " + ((double)(sent * dataToSend.length) / (TimeUtil.currentTimeMillis() - beginTime) * 8 * 1000 / 1024 / 1024) + "MBits/sec");
         // Now it is time to read rest of responses to see how many are lost.
         while (pendingResponsesNumber > 0) 
         {
            icmp.selectSocket(5000);
            responseMsg = icmp.receive();
            if (messageIdSet.remove(new Integer(responseMsg.getMessageId()))) 
            {
               pendingResponsesNumber -= 1;
            };
            if (responseMsg.getStatus() != IcmpStatus.EchoReply) System.out.println("Type: " + responseMsg.getStatus());
            received += 1;
         };
         // Clean up
         System.out.println("ID: " + icmp.getId() + " Pending echo count " + pendingResponsesNumber + " sent: " + sent + " received: " + received);
         icmp.closeSocket();
         icmp = null;
         
         // Increase counter of successfuly executed tests
         successCounter.inc();
      }
      catch (IcmpFatalErrorException ex) 
      {
         ex.printStackTrace();
         failureCounter.inc();
      }
      catch (UnknownHostException ex) 
      {
         ex.printStackTrace();
         failureCounter.inc();
      }
      catch (IcmpSystemException ex) 
      {
         ex.printStackTrace();
         failureCounter.inc();
      }
      catch (IcmpTimeoutException ex) 
      {
         // Just timeout - proceed
         successCounter.inc();
      }
      finally 
      {
         System.out.println("Finally");
         System.out.println(" Pending echo count " + pendingResponsesNumber + " sent: " + sent + " received: " + received);
         if (icmp != null) 
         {
            System.out.println("ID: " + icmp.getId() + " Pending echo count " + pendingResponsesNumber + " sent: " + sent + " received: " + received);
            try 
            {
               icmp.closeSocket();
            }
            catch (IcmpSystemException ex) 
            {
               // ignore it - anyway we exit funtion
            };
            icmp = null;
         }        
      };
      System.out.println("PingHandler.run OUT " + Thread.currentThread().getId());
   }
   
}
